@@ -2,8 +2,7 @@
 # vim:fileencoding=utf-8
 # License: GPLv3 Copyright: 2016, Kovid Goyal <kovid at kovidgoyal.net>
 
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+from __future__ import absolute_import, division, print_function, unicode_literals
 from io import BytesIO
 from itertools import count
 from functools import partial
@@ -15,11 +14,12 @@ from calibre.ebooks.oeb.polish.create import create_book
 from calibre.ebooks.oeb.polish.cover import (
     find_cover_image, mark_as_cover, find_cover_page, mark_as_titlepage, clean_opf
 )
-from calibre.ebooks.oeb.polish.toc import get_toc, from_xpaths as toc_from_xpaths
+from calibre.ebooks.oeb.polish.toc import get_toc, from_xpaths as toc_from_xpaths, get_landmarks
 from calibre.ebooks.oeb.polish.utils import guess_type
 from calibre.ebooks.oeb.base import OEB_DOCS
 from calibre.ebooks.metadata.book.base import Metadata
 from calibre.ebooks.metadata.opf3 import CALIBRE_PREFIX
+from polyglot.builtins import unicode_type
 
 OPF_TEMPLATE = '''
 <package xmlns="http://www.idpf.org/2007/opf" version="{ver}" prefix="calibre: %s" unique-identifier="uid">
@@ -32,10 +32,14 @@ OPF_TEMPLATE = '''
     <guide>{guide}</guide>
 </package>''' % CALIBRE_PREFIX  # noqa
 
+
 def create_manifest_item(name, data=b'', properties=None):
     return (name, data, properties)
+
+
 cmi = create_manifest_item
 
+
 def create_epub(manifest, spine=(), guide=(), meta_cover=None, ver=3):
     mo = []
     for name, data, properties in manifest:
@@ -48,7 +52,7 @@ def create_epub(manifest, spine=(), guide=(), meta_cover=None, ver=3):
     if not spine:
         spine = [x[0] for x in manifest if guess_type(x[0]) in OEB_DOCS]
     spine = ''.join('<itemref idref="%s"/>' % name for name in spine)
-    guide = ''.join('<reference href="%s" type="%s"/>' % (name, typ) for name, typ in guide)
+    guide = ''.join('<reference href="%s" type="%s" title="%s"/>' % (name, typ, title) for name, typ, title in guide)
     opf = OPF_TEMPLATE.format(manifest=mo, ver='%d.0'%ver, metadata=metadata, spine=spine, guide=guide)
     buf = BytesIO()
     with ZipFile(buf, 'w', ZIP_STORED) as zf:
@@ -66,6 +70,7 @@ def create_epub(manifest, spine=(), guide=(), meta_cover=None, ver=3):
     buf.seek(0)
     return buf
 
+
 counter = count()
 
 
@@ -73,7 +78,7 @@ class Structure(BaseTest):
 
     def create_epub(self, *args, **kw):
         n = next(counter)
-        ep = os.path.join(self.tdir, str(n) + 'book.epub')
+        ep = os.path.join(self.tdir, unicode_type(n) + 'book.epub')
         with open(ep, 'wb') as f:
             f.write(create_epub(*args, **kw).getvalue())
         c = get_container(ep, tdir=os.path.join(self.tdir, 'container%d' % n), tweak_mode=True)
@@ -89,7 +94,7 @@ class Structure(BaseTest):
         self.assertEqual(3, c.opf_version_parsed.major)
         self.assertTrue(len(get_toc(c)))  # detect NCX toc even in epub 3 files
         c.add_file('nav.html', b'<html xmlns="http://www.w3.org/1999/xhtml" xmlns:epub="http://www.idpf.org/2007/ops">'
-                   '<body><nav epub:type="toc"><ol><li><a href="start.xhtml">EPUB 3 nav</a></li></ol></nav></body></html>',
+                   b'<body><nav epub:type="toc"><ol><li><a href="start.xhtml">EPUB 3 nav</a></li></ol></nav></body></html>',
                    process_manifest_item=lambda item:item.set('properties', 'nav'))
         toc = get_toc(c)
         self.assertTrue(len(toc))
@@ -118,6 +123,23 @@ class Structure(BaseTest):
         tfx('32123', '321[2[3]]')
         tfx('123123', '1[2[3]]1[2[3]]')
 
+    def test_landmarks_detection(self):
+        c = self.create_epub([cmi('xxx.html'), cmi('a.html')], guide=[('xxx.html#moo', 'x', 'XXX'), ('a.html', '', 'YYY')], ver=2)
+        self.assertEqual(2, c.opf_version_parsed.major)
+        self.assertEqual([
+            {'dest':'xxx.html', 'frag':'moo', 'type':'x', 'title':'XXX'}, {'dest':'a.html', 'frag':'', 'type':'', 'title':'YYY'}
+        ], get_landmarks(c))
+        c = self.create_epub([cmi('xxx.html'), cmi('a.html')], ver=3)
+        self.assertEqual(3, c.opf_version_parsed.major)
+        c.add_file('xxx/nav.html', b'<html xmlns="http://www.w3.org/1999/xhtml" xmlns:epub="http://www.idpf.org/2007/ops">'
+                   b'<body><nav epub:type="landmarks"><ol><li><a epub:type="x" href="../xxx.html#moo">XXX </a></li>'
+                   b'<li><a href="../a.html"> YYY </a></li>'
+                   b'</ol></nav></body></html>',
+                   process_manifest_item=lambda item:item.set('properties', 'nav'))
+        self.assertEqual([
+            {'dest':'xxx.html', 'frag':'moo', 'type':'x', 'title':'XXX'}, {'dest':'a.html', 'frag':'', 'type':'', 'title':'YYY'}
+        ], get_landmarks(c))
+
     def test_epub3_covers(self):
         # cover image
         ce = partial(self.create_epub, ver=3)
@@ -143,7 +165,7 @@ class Structure(BaseTest):
 
         # clean opf of all cover information
         c = ce([cmi('c.jpg', b'z', 'cover-image'), cmi('c.html', b'', 'calibre:title-page'), cmi('d.html')],
-                             meta_cover='c.jpg', guide=[('c.jpg', 'cover'), ('d.html', 'cover')])
+                             meta_cover='c.jpg', guide=[('c.jpg', 'cover', ''), ('d.html', 'cover', '')])
         self.assertEqual(set(clean_opf(c)), {'c.jpg', 'c.html', 'd.html'})
         self.assertFalse(c.opf_xpath('//*/@name'))
         self.assertFalse(c.opf_xpath('//*/@type'))
@@ -157,7 +179,7 @@ class Structure(BaseTest):
         self.assertIsNone(find_cover_image(c))
         c = ce([cmi('c.jpg')], meta_cover='c.jpg')
         self.assertEqual('c.jpg', find_cover_image(c))
-        c = ce([cmi('c.jpg'), cmi('d.jpg')], guide=[('c.jpg', 'cover')])
+        c = ce([cmi('c.jpg'), cmi('d.jpg')], guide=[('c.jpg', 'cover', '')])
         self.assertEqual('c.jpg', find_cover_image(c))
         mark_as_cover(c, 'd.jpg')
         self.assertEqual('d.jpg', find_cover_image(c))
