@@ -10,7 +10,7 @@ from functools import partial
 
 from PyQt5.Qt import Qt, QAction, pyqtSignal
 
-from calibre.constants import isosx, iswindows
+from calibre.constants import isosx, iswindows, plugins
 from calibre.gui2 import (
     error_dialog, Dispatcher, question_dialog, config, open_local_file,
     info_dialog, elided_text)
@@ -18,6 +18,7 @@ from calibre.gui2.dialogs.choose_format import ChooseFormatDialog
 from calibre.utils.config import prefs, tweaks
 from calibre.ptempfile import PersistentTemporaryFile
 from calibre.gui2.actions import InterfaceAction
+from polyglot.builtins import unicode_type
 
 
 class HistoryAction(QAction):
@@ -125,11 +126,11 @@ class ViewAction(InterfaceAction):
                         kwargs=dict(args=args))
             else:
                 if iswindows:
-                    from calibre.utils.file_associations import file_assoc_windows
+                    winutil = plugins['winutil'][0]
                     ext = name.rpartition('.')[-1]
                     if ext:
                         try:
-                            prog = file_assoc_windows(ext)
+                            prog = winutil.file_association(unicode_type('.' + ext))
                         except Exception:
                             prog = None
                         if prog and prog.lower().endswith('calibre.exe'):
@@ -259,7 +260,7 @@ class ViewAction(InterfaceAction):
             except:
                 error_dialog(self.gui, _('Cannot view'),
                     _('This book no longer exists in your library'), show=True)
-                self.update_history([], remove=set([id_]))
+                self.update_history([], remove={id_})
                 continue
 
             title   = db.title(id_, index_is_id=True)
@@ -316,8 +317,15 @@ class ViewAction(InterfaceAction):
             return
 
         if self.gui.current_view() is self.gui.library_view:
-            ids = list(map(self.gui.library_view.model().id, rows))
-            self._view_calibre_books(ids)
+            ids = []
+            m = self.gui.library_view.model().id
+            for r in rows:
+                try:
+                    ids.append(m(r))
+                except Exception:
+                    pass
+            if ids:
+                self._view_calibre_books(ids)
         else:
             paths = self.gui.current_view().model().paths(rows)
             for path in paths:
